@@ -1,4 +1,4 @@
-// Copyright (C) 2011-2018 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2011-2019 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -213,7 +213,7 @@ void Iface::addUnicast(const isc::asiolink::IOAddress& addr) {
                       << " already defined on the " << name_ << " interface.");
         }
     }
-    unicasts_.push_back(OptionalValue<IOAddress>(addr, true));
+    unicasts_.push_back(Optional<IOAddress>(addr));
 }
 
 bool
@@ -244,7 +244,7 @@ Iface::hasAddress(const isc::asiolink::IOAddress& address) const {
 
 void
 Iface::addAddress(const isc::asiolink::IOAddress& addr) {
-    addrs_.push_back(Address(addr, OptionalValueState(true)));
+    addrs_.push_back(Address(addr));
 }
 
 void
@@ -252,7 +252,7 @@ Iface::setActive(const IOAddress& address, const bool active) {
     for (AddressCollection::iterator addr_it = addrs_.begin();
          addr_it != addrs_.end(); ++addr_it) {
         if (address == addr_it->get()) {
-            addr_it->specify(OptionalValueState(active));
+            addr_it->unspecified(!active);
             return;
         }
     }
@@ -264,7 +264,7 @@ void
 Iface::setActive(const bool active) {
     for (AddressCollection::iterator addr_it = addrs_.begin();
          addr_it != addrs_.end(); ++addr_it) {
-        addr_it->specify(OptionalValueState(active));
+        addr_it->unspecified(!active);
     }
 }
 
@@ -272,7 +272,7 @@ unsigned int
 Iface::countActive4() const {
     uint16_t count = 0;
     BOOST_FOREACH(Address addr, addrs_) {
-        if (addr.get().isV4() && addr.isSpecified()) {
+        if (!addr.unspecified() && addr.get().isV4()) {
             ++count;
         }
     }
@@ -346,6 +346,23 @@ IfaceMgr::deleteExternalSocket(int socketfd) {
     }
 }
 
+int
+IfaceMgr::purgeBadSockets() {
+    std::vector<int> bad_fds;
+    for (SocketCallbackInfo s : callbacks_) {
+        errno = 0;
+        if (fcntl(s.socket_, F_GETFD) < 0 && (errno == EBADF)) {
+            bad_fds.push_back(s.socket_);
+        }
+    }
+
+    for (auto bad_fd : bad_fds) {
+        deleteExternalSocket(bad_fd);
+    }
+
+    return (bad_fds.size());
+}
+
 void
 IfaceMgr::deleteAllExternalSockets() {
     callbacks_.clear();
@@ -526,7 +543,7 @@ IfaceMgr::openSockets4(const uint16_t port, const bool use_bcast,
 
         BOOST_FOREACH(Iface::Address addr, iface->getAddresses()) {
             // Skip non-IPv4 addresses and those that weren't selected..
-            if (!addr.get().isV4() || !addr.isSpecified()) {
+            if (addr.unspecified() || !addr.get().isV4()) {
                 continue;
             }
 
@@ -958,7 +975,8 @@ IfaceMgr::send(const Pkt6Ptr& pkt) {
     }
 
     // Assuming that packet filter is not NULL, because its modifier checks it.
-    return (packet_filter6_->send(*iface, getSocket(*pkt), pkt));
+    // The packet filter returns an int but in fact it either returns 0 or throws.
+    return (packet_filter6_->send(*iface, getSocket(*pkt), pkt) == 0);
 }
 
 bool
@@ -971,7 +989,8 @@ IfaceMgr::send(const Pkt4Ptr& pkt) {
     }
 
     // Assuming that packet filter is not NULL, because its modifier checks it.
-    return (packet_filter_->send(*iface, getSocket(*pkt).sockfd_, pkt));
+    // The packet filter returns an int but in fact it either returns 0 or throws.
+    return (packet_filter_->send(*iface, getSocket(*pkt).sockfd_, pkt) == 0);
 }
 
 Pkt4Ptr IfaceMgr::receive4(uint32_t timeout_sec, uint32_t timeout_usec /* = 0 */) {
@@ -1040,6 +1059,11 @@ Pkt4Ptr IfaceMgr::receive4Indirect(uint32_t timeout_sec, uint32_t timeout_usec /
         // signal or for some other reason.
         if (errno == EINTR) {
             isc_throw(SignalInterruptOnSelect, strerror(errno));
+        } else if (errno == EBADF) {
+            int cnt = purgeBadSockets();
+            isc_throw(SocketReadError,
+                      "SELECT interrupted by one invalid sockets, purged "
+                       << cnt << " socket descriptors");
         } else {
             isc_throw(SocketReadError, strerror(errno));
         }
@@ -1139,6 +1163,11 @@ Pkt4Ptr IfaceMgr::receive4Direct(uint32_t timeout_sec, uint32_t timeout_usec /*
         // signal or for some other reason.
         if (errno == EINTR) {
             isc_throw(SignalInterruptOnSelect, strerror(errno));
+        } else if (errno == EBADF) {
+            int cnt = purgeBadSockets();
+            isc_throw(SocketReadError,
+                      "SELECT interrupted by one invalid sockets, purged "
+                       << cnt << " socket descriptors");
         } else {
             isc_throw(SocketReadError, strerror(errno));
         }
@@ -1263,6 +1292,11 @@ IfaceMgr::receive6Direct(uint32_t timeout_sec, uint32_t timeout_usec /* = 0 */ )
         // signal or for some other reason.
         if (errno == EINTR) {
             isc_throw(SignalInterruptOnSelect, strerror(errno));
+        } else if (errno == EBADF) {
+            int cnt = purgeBadSockets();
+            isc_throw(SocketReadError,
+                      "SELECT interrupted by one invalid sockets, purged "
+                       << cnt << " socket descriptors");
         } else {
             isc_throw(SocketReadError, strerror(errno));
         }
@@ -1366,6 +1400,11 @@ IfaceMgr::receive6Indirect(uint32_t timeout_sec, uint32_t timeout_usec /* = 0 */
         // signal or for some other reason.
         if (errno == EINTR) {
             isc_throw(SignalInterruptOnSelect, strerror(errno));
+        } else if (errno == EBADF) {
+            int cnt = purgeBadSockets();
+            isc_throw(SocketReadError,
+                      "SELECT interrupted by one invalid sockets, purged "
+                       << cnt << " socket descriptors");
         } else {
             isc_throw(SocketReadError, strerror(errno));
         }
